header {
	package edu.usfca.syndiag;
	import java.awt.*;
}

/** A tree grammar draw syntax diagram
 *
 *  Jia Zheng
 *  University of San Francisco
 *  Oct. 1, 2003
 */


class Draw extends TreeParser;

options {
	importVocab = ANTLR;
    codeGenBitsetTestThreshold=999;
}

{
	SyntaxDiagramGenerator diagGen = null;
	
	public void setDiagGen(SyntaxDiagramGenerator diagGen){
		this.diagGen = diagGen;
	}
}

grammar 
    :   classDef // ignore 2nd (lexer) grammar for now
    ;

classDef
    :    rule 
    |   #( PARSER rules )
    |   #( TREE_PARSER rules )
    |   #( LEXER rules )
    ;

rules
    :   ( rule )+
    ;

rule
	{GrammarAST rl = (GrammarAST)#rule;
	int x; int y;
	int x1 = ((GrammarAST)rl).getX();
    int y1 = ((GrammarAST)rl).getY();
    int h;}
    :   #( RULE r:RULE_REF 
    			{x = ((GrammarAST)r).getX();
    			 y = ((GrammarAST)r).getY();
    			 if (rl.signal != 1){
    			 	diagGen.beginRule(r.getText(), x, y);
    			 }else{
    			 	diagGen.continueRule(r.getText(), x, y);
    			 }
    			}
    		#(BLOCK h=b:block[BLOCK]
    			{int x2 = ((GrammarAST)b).getX();
    			 int y2 = ((GrammarAST)b).getY();}
    		) EOR ) 
    		{int w = ((GrammarAST)rl).getWidth();
    		 diagGen.endRule(x1, y1, x2, y2, w, rl.signal);
    		}
    ;

block[int type] returns [int h]
	{GrammarAST b = (GrammarAST)#block;
	int[] leftConnPoint, rightConnPoint;
	int[] connPoint1 = ((GrammarAST)b).getBlockLeftConnPoint();
	int[] connPoint2 = ((GrammarAST)b).getBlockRightConnPoint();
	int i=1;
	h = ((GrammarAST)b).getBlockHeight();
	}
    :   ( a:alternative
    		{leftConnPoint = ((GrammarAST)a).getLeftConnPoint();
    		 rightConnPoint = ((GrammarAST)a).getRightConnPoint();
    		 diagGen.drawBlock(connPoint1, connPoint2, leftConnPoint, rightConnPoint, i);
    		 i++;
    		}
    	)+
    ;

alternative
	{int i = 1;
	 int[] leftConnPoint, rightConnPoint;
	 }
    :   #( a:ALT
    		{rightConnPoint = ((GrammarAST)a).getRightConnPoint();
    		}
    			 (e:element[ALT]
    				{if (i > 1)
    					{leftConnPoint = ((GrammarAST)e).getLeftConnPoint();
    				 	 diagGen.drawArrow(rightConnPoint[0], rightConnPoint[1], leftConnPoint[0], leftConnPoint[1]);
    				 	 }
    				 rightConnPoint = ((GrammarAST)e).getRightConnPoint();
    				 i++;
    				}
    			)+ )
    ;

element [int type]
	{GrammarAST e = (GrammarAST)#element;
	int hb;}
    :   atom[type, 1]
    |   #(NOT element[NOT]) 
    |   #(TOKEN_RANGE atom[TOKEN_RANGE, 1] atom[TOKEN_RANGE, 2])
    |   #(CHAR_RANGE CHAR_LITERAL CHAR_LITERAL)
    |   ebnf
    |   tree
    |   #( s:SYNPRED hb=block[SYNPRED] )
    		{int[] point1 = ((GrammarAST)s).getLeftConnPoint();
    		 int[] point2 = ((GrammarAST)s).getRightConnPoint();
    		 int x = point1[0] - 5;
    		 int y = point1[1] - 25;
    		 int w = point2[0] - point1[0] + 10 + 5;
    		 int h = ((GrammarAST)s).getHeight() + 30;
    		 diagGen.drawDotRect(x, y, w, h );
    	 }
    |   ACTION
    |   SEMPRED
    |   EPSILON
    ;

ebnf
	{GrammarAST e = (GrammarAST)#ebnf;
	int h1, h3, h2, h;
	}
	:   #( BLOCK h1=block[BLOCK] )
    |   #( OPTIONAL h3=b3:block[OPTIONAL] )
    		{int[] connPoint1 = ((GrammarAST)b3).getBlockLeftConnPoint();
	 		 int[] connPoint2 = ((GrammarAST)b3).getBlockRightConnPoint();
	 		 diagGen.drawOptional(connPoint1, connPoint2, 15);
    		}
    |   #( CLOSURE h=b:block[CLOSURE] )
    		{int[] connPoint1 = ((GrammarAST)b).getBlockLeftConnPoint();
	 		 int[] connPoint2 = ((GrammarAST)b).getBlockRightConnPoint();
	 		 diagGen.drawClosure(connPoint1, connPoint2, h);
    		}
    |   #( POSITIVE_CLOSURE h2=b2:block[POSITIVE_CLOSURE] )
    		{int[] connPoint1 = ((GrammarAST)b2).getBlockLeftConnPoint();
	 		 int[] connPoint2 = ((GrammarAST)b2).getBlockRightConnPoint();
	 		 diagGen.drawPClosure(connPoint1, connPoint2, h2);
			}
    ;

tree
	{int[] leftPoint = new int[2];
	 int[] rightPoint = new int[2];
	}
	:   #(TREE_BEGIN a:atom[0, 1]
						{rightPoint = ((GrammarAST)a).getRightConnPoint();}
						(e:element[TREE_BEGIN]
							{leftPoint = ((GrammarAST)e).getLeftConnPoint();
							 diagGen.drawArrow(rightPoint[0], rightPoint[1], 
										leftPoint[0], leftPoint[1]);
							 rightPoint = ((GrammarAST)e).getRightConnPoint();
							}
						)* )
    ;

atom [int type, int pos]
	{GrammarAST a = (GrammarAST)#atom;
	int x = a.getX();
	int y = a.getY();
	int w = a.getWidth();
	int h = a.getHeight();
	}
    :   r:RULE_REF
    		{diagGen.drawRuleRef(x, y, w, h, r.getText(), new Font("Monospaces", Font.PLAIN, 15));}
    |   t:TOKEN_REF
    		{if (type == NOT){
    		 	diagGen.drawNot(x, y, w, h, "~ " + t.getText(), new Font("Monospaces", Font.PLAIN, 15));
    		 }else if (type == TOKEN_RANGE){
    		 	diagGen.drawRange(x, y, w, h, t.getText(), pos);
    		 }else{
    		 	diagGen.drawTokenRef(x, y, w, h, t.getText(), new Font("Monospaces", Font.PLAIN, 15));
    		 }
    		}
    |   c:CHAR_LITERAL
    		{diagGen.drawL(x, y, w, h, c.getText(), new Font("Monospaces", Font.PLAIN, 15));}
    |   s:STRING_LITERAL
    		{diagGen.drawL(x, y, w, h, s.getText(), new Font("Monospaces", Font.PLAIN, 15));}
    |   wi:WILD_CARD
    		{diagGen.drawL(x, y, w, h, wi.getText(), new Font("Monospaces", Font.PLAIN, 15));}
    ;

